X-Git-Url: http://git.vanrenterghem.biz/git.ikiwiki.info.git/blobdiff_plain/930584cff747d87635da192585030d4fb30a2893..b4ab0d0b9eae7b2204b8ea8e1ee70e63117b23ab:/IkiWiki/Plugin/map.pm

diff --git a/IkiWiki/Plugin/map.pm b/IkiWiki/Plugin/map.pm
index b08d78d76..18c584a30 100644
--- a/IkiWiki/Plugin/map.pm
+++ b/IkiWiki/Plugin/map.pm
@@ -12,9 +12,18 @@ use strict;
 use IkiWiki 2.00;
 
 sub import { #{{{
+	hook(type => "getsetup", id => "map", call => \&getsetup);
 	hook(type => "preprocess", id => "map", call => \&preprocess);
 } # }}}
 
+sub getsetup () { #{{{
+	return
+		plugin => {
+			safe => 1,
+			rebuild => undef,
+		},
+} #}}}
+
 sub preprocess (@) { #{{{
 	my %params=@_;
 	$params{pages}="*" unless defined $params{pages};
@@ -25,8 +34,14 @@ sub preprocess (@) { #{{{
 	my %mapitems;
 	foreach my $page (keys %pagesources) {
 		if (pagespec_match($page, $params{pages}, location => $params{page})) {
-			$mapitems{$page}=1;
-
+			if (exists $params{show} && 
+			    exists $pagestate{$page} &&
+			    exists $pagestate{$page}{meta}{$params{show}}) {
+				$mapitems{$page}=$pagestate{$page}{meta}{$params{show}};
+			}
+			else {
+				$mapitems{$page}='';
+			}
 			# Check for a common prefix.
 			if (! defined $common_prefix) {
 				$common_prefix=$page;
@@ -37,6 +52,9 @@ sub preprocess (@) { #{{{
 				my @b=split(/\//, $common_prefix);
 				$common_prefix="";
 				while (@a && @b && $a[0] eq $b[0]) {
+					if (length $common_prefix) {
+						$common_prefix.="/";
+					}
 					$common_prefix.=shift(@a);
 					shift @b;
 				}
@@ -50,8 +68,9 @@ sub preprocess (@) { #{{{
 		$common_prefix=IkiWiki::dirname($common_prefix);
 	}
 
 	# Needs to update whenever a page is added or removed, so
 	# register a dependency.
 	add_depends($params{page}, $params{pages});
 	# Explicitly add all currently shown pages, to detect when pages
 	# are removed.
@@ -61,23 +80,24 @@ sub preprocess (@) { #{{{
 	my $parent="";
 	my $indent=0;
 	my $openli=0;
-	my $dummy=0;
+	my $addparent="";
 	my $map = "<div class='map'>\n<ul>\n";
 	foreach my $item (sort keys %mapitems) {
+		my @linktext = (length $mapitems{$item} ? (linktext => $mapitems{$item}) : ());
 		$item=~s/^\Q$common_prefix\E\///
 			if defined $common_prefix && length $common_prefix;
 		my $depth = ($item =~ tr/\//\//) + 1;
 		my $baseitem=IkiWiki::dirname($item);
 		while (length $parent && length $baseitem && $baseitem !~ /^\Q$parent\E(\/|$)/) {
 			$parent=IkiWiki::dirname($parent);
-			last if !$dummy && length $parent && $baseitem =~ /^\Q$parent\E(\/|$)/;
+			last if length $addparent && $baseitem =~ /^\Q$addparent\E(\/|$)/;
+			$addparent="";
 			$indent--;
 			$map .= "</li>\n";
 			if ($indent > 0) {
 				$map .= "</ul>\n";
 			}
 		}
-		$dummy=0;
 		while ($depth < $indent) {
 			$indent--;
 			$map .= "</li>\n";
@@ -94,10 +114,13 @@ sub preprocess (@) { #{{{
 				$map .= "<ul>\n";
 			}
 			if ($depth > $indent) {
-				$dummy=1;
 				$p.="/".shift(@bits);
+				$addparent=$p;
+				$addparent=~s/^\///;
 				$map .= "<li>"
-					.htmllink($params{page}, $params{destpage}, $p, class => "mapparent")
+					.htmllink($params{page}, $params{destpage},
+						 "/".$common_prefix.$p, class => "mapparent",
+						 noimageinline => 1)
 					."\n";
 				$openli=1;
 			}
@@ -108,7 +131,9 @@ sub preprocess (@) { #{{{
 		$map .= "</li>\n" if $openli;
 		$map .= "<li>"
 			.htmllink($params{page}, $params{destpage}, 
-				"/".$common_prefix."/".$item, class => "mapitem")
+				"/".$common_prefix."/".$item,
+				@linktext,
+				class => "mapitem", noimageinline => 1)
 			."\n";
 		$openli=1;
 		$parent=$item;
@@ -50,8 +68,9 @@ sub preprocess (@) { #{{{
 	}
 
-	# Needs to update whenever a page is added or removed, so
-	# register a dependency.
+	# Needs to update whenever a page is added or removed (or in some
+	# cases, when its content changes, if show=title), so register a
+	# dependency.
 	add_depends($params{page}, $params{pages});
 	# Explicitly add all currently shown pages, to detect when pages